package com.rjconsultores.ventaboletos.relatorios.impl;

import java.sql.Connection;
import java.util.EnumSet;
import java.util.Map;

import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;

public class RelatorioLinhaOperacional extends Relatorio {

	public enum EnumEspecie {
		BILHETE("BILHETE DE PASSAGEM RODOVIRIO"), EXCESSO_BAGAGEM("DEMOSTRATIVO EXCESSO DE BAGAGEM");
		public static final EnumSet<EnumEspecie> all = EnumSet.of(BILHETE, EXCESSO_BAGAGEM);
		private String descricao;

		private EnumEspecie(String descricao) {
			this.descricao = descricao;
		}

		@Override
		public String toString() {
			return descricao;
		}
	}

	public RelatorioLinhaOperacional(Map<String, Object> parametros, Connection conexao) {
		super(parametros, conexao);
	}

	@Override
	protected void processaParametros() throws Exception {
		Map<String, Object> parametros = this.getParametros();

		EnumEspecie especie = (EnumEspecie) parametros.get("ESPECIE");
		String dataDe = (String) parametros.get("DATA_DE");
		String dataAte = (String) parametros.get("DATA_ATE");
		String linhasIds = (String) parametros.get("LINHAS");
		String empresaId = (String) parametros.get("EMPRESA_IDS");

		String sql = getSql(especie, dataDe, dataAte, linhasIds, empresaId);
		parametros.put("SQL", sql);
		parametros.put("STR_ESPECIE", especie.toString());
	}

	private String getSql(EnumEspecie especie, String dataDe, String dataAte, String linhasIds, String empresaId) {
		StringBuilder sql = new StringBuilder();

		sql.append(" SELECT c.feccorrida   data_corrida, ");
		sql.append("       c.fechorsalida hora_saida, ");
		sql.append("       r.descruta     linha, ");
		sql.append("       o.descparada   origem, ");
		sql.append("       d.descparada   destino, ");
		sql.append("       e.empresa_id   empresaId, ");
		sql.append("       e.nombempresa  empresa, ");
		sql.append("       Count(*)       qtde_boletos ");

		switch (especie) {
		case BILHETE:
			sql.append(" FROM  caja ca ");
			sql.append("       INNER JOIN corrida c ON ( c.corrida_id = ca.corrida_id AND c.feccorrida = ca.feccorrida ) ");
			sql.append("       INNER JOIN empresa e ON e.empresa_id = c.empresacorrida_id ");
			sql.append("       INNER JOIN ruta r ON r.ruta_id = c.ruta_id ");
			sql.append("       INNER JOIN parada o ON o.parada_id = ca.origen_id ");
			sql.append("       INNER JOIN parada d ON d.parada_id = ca.destino_id ");
			sql.append(" WHERE  ca.fechorventa BETWEEN To_date('").append(dataDe).append("', 'dd/mm/yyyy') ");
			sql.append("       AND To_date('").append(dataAte).append("', 'dd/mm/yyyy') ");
			sql.append("       AND c.ruta_id IN (").append(linhasIds).append(") ");
			sql.append("       AND ca.empresacorrida_id IN (").append(empresaId).append(") ");
			break;
		case EXCESSO_BAGAGEM:
			sql.append(" FROM  caja_diversos ca ");
			sql.append("       INNER JOIN evento_extra ex ON ex.eventoextra_id = ca.eventoextra_id ");
			sql.append("       INNER JOIN corrida c ON ( c.corrida_id = ex.corrida_id AND c.feccorrida = ex.feccorrida ) ");
			sql.append("       INNER JOIN empresa e ON e.empresa_id = c.empresacorrida_id ");
			sql.append("       INNER JOIN ruta r ON r.ruta_id = c.ruta_id ");
			sql.append("       INNER JOIN parada o ON o.parada_id = c.origen_id ");
			sql.append("       INNER JOIN parada d ON d.parada_id = c.destino_id ");
			sql.append(" WHERE  ca.fechorvta BETWEEN To_date('").append(dataDe).append("', 'dd/mm/yyyy') ");
			sql.append("       AND To_date('").append(dataAte).append("', 'dd/mm/yyyy') ");
			sql.append("       AND c.ruta_id IN (").append(linhasIds).append(") ");
			sql.append("       AND ex.empresa_id IN (").append(empresaId).append(") ");
			break;
		}

		sql.append(" GROUP BY c.feccorrida, c.fechorsalida, r.descruta, o.descparada, d.descparada, e.empresa_id, e.nombempresa ");
		sql.append(" ORDER BY c.feccorrida, e.nombempresa ASC ");

		return sql.toString();
	}
}
