package com.rjconsultores.ventaboletos.relatorios.utilitarios;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.rjconsultores.ventaboletos.relatorios.render.RenderRelatorioJasper;

/**
 * 
 * Enum complexo contendo as configuraes dos relatrios disponveis no sistema.
 * 
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
/**
 * @author Bruno H. G. Gouva <bruno@rjconsultores.com.br>
 * 
 */
public abstract class Relatorio {

	/*
	 * RELATORIO_APROVEITAMENTO(1, "RelatorioAproveitamento", "Relatrio de Aproveitamento",new RendimentoParamsImpl(), null); RELATORIO_RECEITA_DIARIA_AGENCIA(2, "RelatorioReceitaDiariaAgencia", "Relatrio de Receita Diria por Agncia", null, null), RELATORIO_EXEMPLO(99, "ExemploFoo", "Relatrio Foo", null, null);
	 */
	private Connection conexao;

	private Map<String, Object> parametros;
	protected RenderRelatorioJasper render;
	private IDataSource customDataSource;
	private IParametros processadorParametros;
	private Set<String> infoMsg;

	protected Relatorio(Map<String, Object> parametros, Connection conexao) {

		this.parametros = parametros;
		this.conexao = conexao;
		this.infoMsg = new HashSet<String>();
	}

	public Connection getConexao() {
		return conexao;
	}

	public void setConexao(Connection conexao) {
		this.conexao = conexao;
	}

	public String getNome() {
		return this.getClass().getSimpleName();
	}

	public boolean isCustomDataSource() {
		return this.customDataSource != null;

	}

	public Map<String, Object> getParametros() {
		return parametros;
	}

	/**
	 * @return the customDataSource
	 */
	public IDataSource getCustomDataSource() {
		return customDataSource;
	}

	/**
	 * @param iDataSource
	 *            the customDataSource to set
	 */
	public void setCustomDataSource(IDataSource iDataSource) {
		this.customDataSource = iDataSource;
	}

	public IParametros getProcessadorParametros() {
		return processadorParametros;
	}

	public void setProcessadorParametros(IParametros processadorParametros) {
		this.processadorParametros = processadorParametros;
	}

	protected abstract void processaParametros() throws Exception;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.rjconsultores.ventaboletos.relatorios.utilitarios.IRenderRelatorio#getConteudo()
	 */
	public byte[] getConteudo(SaidaRelatorio saida) throws Exception {
		this.processaParametros();

		if (this.render == null)
			this.render = new RenderRelatorioJasper(this);

		return this.render.render(saida);

	}

	public Set<String> getInfoMsg() {
		return infoMsg;
	}

	public void addInfoMsg(String msg) {
		this.infoMsg.add(msg);
	}

}