package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Bandbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioTrechoVendido;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPuntoVentaSimple;
import com.trg.search.Filter;

@Controller("relatorioTrechoVendidoController")
@Scope("prototype")
public class RelatorioTrechoVendidoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(RelatorioTrechoVendidoController.class);

	@Autowired
	private DataSource dataSource;

	@Autowired
	private EmpresaService empresaService;

	private ArrayList<String> lsNumPuntoVenta = new ArrayList<String>();
	private ArrayList<String> lsNumEmpresa = new ArrayList<String>();

	@Autowired
	private transient PagedListWrapper<PuntoVenta> plwPuntoVenta;
	private MyListbox puntoVentaList;
	private Paging pagingPuntoVenta;
	private Bandbox bbPesquisaPuntoVenta;
	private Textbox txtPalavraPesquisa;
	private Datebox datInicial;
	private Datebox datFinal;

	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresa;

	private void executarRelatorio() throws Exception {

		if (lsNumPuntoVenta.size() > 0) {
			Map<String, Object> parametros = new HashMap<String, Object>();
			parametros.put("DATA_INICIO", new java.sql.Date(((java.util.Date) this.datInicial.getValue()).getTime()));
			parametros.put("DATA_FINAL", new java.sql.Date(((java.util.Date) this.datFinal.getValue()).getTime()));

			if (lsNumPuntoVenta.size() > 0) {
				Integer puntuVentaId = null;
				try {
					puntuVentaId = Integer.parseInt(lsNumPuntoVenta.iterator().next());
				} catch (Exception e) {
					log.error(e.getMessage());
				}
				parametros.put("PUNTOVENTA_ID", puntuVentaId);
			}

			String empresa = "";
			try {
				Empresa e = (Empresa) cmbEmpresa.getSelectedItem().getValue();
				if (!e.equals(null)) {
					empresa = e.getNombempresa();
				}
			} catch (NullPointerException e) {
				empresa = "";
			} catch (Exception e) {
				log.error(e.getMessage());
			}
			parametros.put("NOMBEMPRESA", empresa);

			String empresaSearch = "%" + empresa + "%";
			parametros.put("NOMBEMPRESA_SEARCH", empresaSearch);

			Date now = new Date();
			parametros.put("NOW", now);

			Relatorio relatorio = new RelatorioTrechoVendido(parametros, dataSource.getConnection());

			Map args = new HashMap();
			args.put("relatorio", relatorio);

			openWindow("/component/reportView.zul",
					Labels.getLabel("relatorioTrechoVendidoController.window.title"), args, MODAL);
		} else {
			try {
				Messagebox.show(Labels.getLabel("relatorioTrechoVendidoController.no.agencia"),
						Labels.getLabel("relatorioTrechoVendidoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onSelect$puntoVentaList(Event ev) {

		String strListPuntoVenta = new String();
		ArrayList<Object> lsSelecionados = (ArrayList<Object>) puntoVentaList.getSelectedsItens();

		lsNumPuntoVenta.clear();
		for (Object objPuntoVenta : lsSelecionados) {
			strListPuntoVenta = ((PuntoVenta) objPuntoVenta).getNombpuntoventa() + " " + strListPuntoVenta;
			lsNumPuntoVenta.add(((PuntoVenta) objPuntoVenta).getPuntoventaId().toString());
		}
		this.bbPesquisaPuntoVenta.setValue(strListPuntoVenta);
	}

	public void onClick$btnLimpar(Event ev) {
		limparPesquisaAgencia();
	}

	public void onClick$btnPesquisa(Event ev) {
		executarPesquisa();
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	private void limparPesquisaAgencia() {
		puntoVentaList.clearSelection();

		lsNumPuntoVenta.clear();
		this.bbPesquisaPuntoVenta.setValue("");
	}

	private void executarPesquisa() {
		HibernateSearchObject<PuntoVenta> puntoVentaBusqueda =
				new HibernateSearchObject<PuntoVenta>(PuntoVenta.class,
						pagingPuntoVenta.getPageSize());

		puntoVentaBusqueda.addFilterOr(Filter.like("nombpuntoventa", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")), Filter.like("numPuntoVenta", "%" + txtPalavraPesquisa.getText().trim().toUpperCase().concat("%")));

		puntoVentaBusqueda.addSortAsc("nombpuntoventa");

		puntoVentaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwPuntoVenta.init(puntoVentaBusqueda, puntoVentaList, pagingPuntoVenta);

		if (puntoVentaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("relatorioTrechoVendidoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		puntoVentaList.setItemRenderer(new RenderPuntoVentaSimple());
		lsEmpresa = empresaService.obtenerTodos();
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public ArrayList<String> getLsNumPuntoVenta() {
		return lsNumPuntoVenta;
	}

	public void setLsNumPuntoVenta(ArrayList<String> lsNumPuntoVenta) {
		this.lsNumPuntoVenta = lsNumPuntoVenta;
	}

	public ArrayList<String> getLsNumEmpresa() {
		return lsNumEmpresa;
	}

	public void setLsNumEmpresa(ArrayList<String> lsNumEmpresa) {
		this.lsNumEmpresa = lsNumEmpresa;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

}
